local perFrame, perFrame2
local perTime, perTime2
local size
local width
local vsync
local msaa
local last
local ravg
local avgp
local avgl
local fscr
local fsmd

function love.load()

	local w,h,flags = love.window.getMode()

	perFrame = {x = 0, y = 100}
	perTime  = {x = 0, y = 200, rr = 1/flags.refreshrate or 1/60}
	size  = 100

	width = love.graphics.getWidth()
	vsync = flags.vsync
	msaa  = flags.msaa == 0 and 5 or math.log(flags.msaa)/math.log(2)
	fscr  = flags.fullscreen
	fsmd  = flags.fullscreentype

	last = 0.0
	ravg = {}
	avgp = 0
	avgl = 4

	for i=0,avgl-1 do ravg[i] = 0.0 end

	perFrame2 = {x = 0, y = 300}
	perTime2  = {x = 0, y = 400, rr = 1/flags.refreshrate or 1/60}

end

function love.update(dt)
	avgp = (avgp + 1) % avgl
	ravg[avgp] = dt-last

	--dt = math.min(dt, perTime.rr)

	perFrame.x = (perFrame.x +   0.5     ) % (width+size)
    perTime.x  = (perTime.x  + 500.0 * dt) % (width+size)

    perFrame2.x = (perFrame2.x +   8.36     ) % (width+size)
    perTime2.x  = (perTime2.x  +   5.0 * dt) % (width+size)

    last = dt
end

function love.draw()
	love.graphics.setColor(63,63,91)
	love.graphics.rectangle("fill", 0, 0, width, 100)

	love.graphics.setColor(255,255,255)
	love.graphics.print("Press R to reset blocks.", 0, 0)
	love.graphics.print(string.format("Press V to toggle vsync; currently: %s",vsync and "ON" or "OFF"), 0, 12)
	love.graphics.print(string.format("Press M to toggle msaa; current lv: %d",(2^msaa)%(2^5)), 0, 24)
	love.graphics.print(string.format("DeltaTime: %q",love.timer.getDelta()), 0, 36)

	local avg = 0.0
	for i=0,avgl-1 do
		avg = avg + ravg[i]
	end
	avg = (avg / avgl)
	love.graphics.print(string.format("dt fluctuation (%d values): %f", avgl, math.abs(avg)), 0, 48)
	love.graphics.print(string.format("direction: %s",(avg < 0 and "-" or "+")), 0, 60)

	love.graphics.print(string.format("Press F to toggle fullscreen, T to toggle mode; currently: %s (%s mode)",fscr and "ON" or "OFF",fsmd), 0, 72)
	love.graphics.print("Press Q to quit.", 0, 84)

	love.graphics.rectangle("fill", perFrame.x-size, perFrame.y, 100, 100)
    love.graphics.rectangle("fill", perTime.x-size,  perTime.y,  100, 100)
    love.graphics.rectangle("fill", perFrame2.x-size, perFrame2.y, 100, 100)
    love.graphics.rectangle("fill", perTime2.x-size,  perTime2.y,  100, 100)
end

function love.keyreleased(key)

	if key == 'r' then
		local r = perTime.rr
		perFrame = {x = 0, y = 100}
		perTime  = {x = 0, y = 200, rr = r}
		perFrame2 = {x = 0, y = 300}
		perTime2  = {x = 0, y = 400, rr = r}
	end

	if key == 'v' then
		local w,h,flags = love.window.getMode()
		flags.vsync = not flags.vsync
		vsync = flags.vsync
		love.window.setMode(w,h,flags)
		width = w
	end

	if key == 'm' then
		local w,h,flags = love.window.getMode()
		msaa = (msaa + 1) % 6
		flags.msaa = (2^msaa)%(2^5)
		love.window.setMode(w,h,flags)
		width = w
	end

	if key == 'f' then
		local w,h,flags = love.window.getMode()
		flags.fullscreen = not flags.fullscreen
		fscr = not fscr
		love.window.setMode(w,h,flags)
		width = w
	end

	if key == 't' then
		local w,h,flags = love.window.getMode()
		flags.fullscreentype = flags.fullscreentype == 'desktop' and 'exclusive' or 'desktop'
		fsmd = fsmd == 'desktop' and 'exclusive' or 'desktop'
		love.window.setMode(w,h,flags)
		width = w
	end

	if key == 'q' then love.event.quit() end

end

function love.run()
 
	if love.math then
		love.math.setRandomSeed(os.time())
	end
 
	if love.load then love.load(arg) end
 
	-- We don't want the first frame's dt to include time taken by love.load.
	if love.timer then love.timer.step() end
 
	local dt = 0
 
	-- Main loop time.
	while true do
		-- Process events.
		if love.event then
			love.event.pump()
			for name, a,b,c,d,e,f in love.event.poll() do
				if name == "quit" then
					if not love.quit or not love.quit() then
						return a
					end
				end
				love.handlers[name](a,b,c,d,e,f)
			end
		end
 
		-- Update dt, as we'll be passing it to update
		if love.timer then
			love.timer.step()
			dt = love.timer.getDelta()
		end
 
		-- Call update and draw
		if love.update then love.update(dt) end -- will pass 0 if love.timer is disabled
 
		if love.graphics and love.graphics.isActive() then
			love.graphics.clear(love.graphics.getBackgroundColor())
			love.graphics.origin()
			if love.draw then love.draw() end
			love.graphics.present()
		end
 
		if love.timer and not vsync then love.timer.sleep(0.001) end
	end
 
end